#include <sys/qioctl.h>
#include <sys/cdrom.h>

#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>

static int cd;

int
match(char *token, char *word, int min) {
    return strnicmp(token, word, min(min, strlen(word))) == 0;
}

int
parse(char *line) {
    char *p = strtok(line, "\t\n ,");
    int ret;

    if (match(p, "play", 2)) {
        struct cdrom_ti ti;

        memset(&ti, 0, sizeof ti);
        if (p = strtok(0, "\t\n ,"))
            ti.cdti_trk0 = strtol(p, &p, 10);
        else
            ti.cdti_trk0 = 1;

        if (p && *p == ':')
            ti.cdti_ind0 = atoi(++p);
        else
            ti.cdti_ind0 = 1;

        if (p && (p = strtok(0, "\t\n ,")))
            ti.cdti_trk1 = strtol(p, &p, 10);
        else
            ti.cdti_trk1 = 99;

        if (p && *p == ':')
            ti.cdti_ind1 = atoi(++p);
        else
            ti.cdti_ind1 = 99;

        printf("Start %d:%d  End %d:%d\n",
            ti.cdti_trk0, ti.cdti_ind0,
            ti.cdti_trk1, ti.cdti_ind1);
        ret = ioctl(cd, CDROMPLAYTRKIND, &ti);
        if(ret == -1)
            perror("PLAYTRKIND");
        else
            printf("ret = %d\n");
    } else if (match(p, "msf", 2)) {
        struct cdrom_msf msf;

        memset(&msf, 0, sizeof msf);
        if (p = strtok(0, "\t\n ,"))
            {
            msf.cdmsf_min0   = strtol(p, &p, 10);
            if (p && *p == ':')
                {
                msf.cdmsf_sec0   = strtol(++p, &p, 10);
                if (p && *p == ':')
                    msf.cdmsf_frame0 = strtol(++p, &p, 10);
                }
            }
        else
            {
            msf.cdmsf_min0   = 0;
            msf.cdmsf_sec0   = 0;
            msf.cdmsf_frame0 = 0;
            }

        if (p && (p = strtok(0, "\t\n ,")))
            {
            msf.cdmsf_min1   = strtol(p, &p, 10);
            if (p && *p == ':')
                {
                msf.cdmsf_sec1   = strtol(++p, &p, 10);
                if (p && *p == ':')
                    msf.cdmsf_frame1 = strtol(++p, &p, 10);
                }
            }
        else
            {
            msf.cdmsf_min1   = 99;
            msf.cdmsf_sec1   = 59;
            msf.cdmsf_frame1 = 74;
            }

        printf("Start %u:%u:%u  End %u:%u:%u\n",
            msf.cdmsf_min0, msf.cdmsf_sec0, msf.cdmsf_frame0,
            msf.cdmsf_min1, msf.cdmsf_sec1, msf.cdmsf_frame1);
        ret = ioctl(cd, CDROMPLAYMSF, &msf);
        if(ret == -1)
            perror("PLAYMSF");
        else
            printf("ret = %d\n");
    } else if (match(p, "pause", 2))
        ret = ioctl(cd, CDROMPAUSE);
    else if (match(p, "resume", 1))
        ret = ioctl(cd, CDROMRESUME);
    else if (match(p, "start", 3))
        ret = ioctl(cd, CDROMSTART);
    else if (match(p, "stop", 3))
        ret = ioctl(cd, CDROMSTOP);
    else if (match(p, "eject", 2))
        ret = ioctl(cd, CDROMEJECT);
    else if (match(p, "hdr", 1)) {
        struct cdrom_tochdr th;

        memset(&th, 0, sizeof th);
#if 1
        ret = ioctl(cd, CDROMREADTOCHDR, &th);
#else
//      ret = qnx_ioctl(cd, CDROMREADTOCHDR, &th, sizeof th, &th, sizeof th);
        ret = qnx_ioctl(cd, CDROMREADTOCHDR, 0, 0, &th, sizeof th);
#endif
        printf("First=%d last=%d\n", th.cdth_trk0, th.cdth_trk1);
    } else if (match(p, "entry", 2)) {
        struct cdrom_tocentry te;

        memset(&te, 0, sizeof te);
        if (p = strtok(0, "\t\n "))
            te.cdte_track = strtol(p, &p, 10);
        else
            te.cdte_track = 0;

        if (p && *p == ',')
            te.cdte_format = atoi(++p);
        else
            te.cdte_format = CDROM_MSF;

        printf("Read Header track=%d format=%d\n", te.cdte_track, te.cdte_format);
#if 1
        ret = ioctl(cd, CDROMREADTOCENTRY, &te);
#else
        ret = qnx_ioctl(cd, CDROMREADTOCENTRY, &te, sizeof te, &te, sizeof te);
#endif
        if(te.cdte_format == CDROM_LBA)
            printf("Track=%d adr=%d ctrl=%d format=%d lba=%lx datamode=%d\n",
                te.cdte_track,
                te.cdte_adr,
                te.cdte_ctrl,
                te.cdte_format,
                te.cdte_addr.lba,
                te.cdte_datamode);
        else
            printf("Track=%d adr=%d ctrl=%d format=%d msf=%u:%u:%u datamode=%d\n",
                te.cdte_track,
                te.cdte_adr,
                te.cdte_ctrl,
                te.cdte_format,
                te.cdte_addr.msf.minute,
                te.cdte_addr.msf.second,
                te.cdte_addr.msf.frame,
                te.cdte_datamode);
    } else if (match(p, "sub", 2)) {
        struct cdrom_subchnl sc;

        memset(&sc, 0, sizeof sc);
        if (p = strtok(0, "\t\n "))
            sc.cdsc_format = strtol(p, &p, 10);
        else
            sc.cdsc_format = CDROM_MSF;

        printf("Read Sub-Channel format=%d\n", sc.cdsc_format);
#if 0
        ret = ioctl(cd, CDROMSUBCHNL, &sc);
#else
        ret = qnx_ioctl(cd, CDROMSUBCHNL, &sc, sizeof sc, &sc, sizeof sc);
#endif
        printf("fmt=%d astat=0x%02x adr=%d ctrl=%d trk=%d ind=%d abs=%lx rel=%lx\n",
            sc.cdsc_format,
            sc.cdsc_audiostatus,
            sc.cdsc_adr,
            sc.cdsc_ctrl,
            sc.cdsc_trk,
            sc.cdsc_ind,
            sc.cdsc_absaddr.lba,
            sc.cdsc_reladdr.lba);
    } else if (match(p, "volume", 1)) {
        struct cdrom_volctrl vc;

        if (p = strtok(0, "\t\n ")) {
            vc.channel0 =
            vc.channel1 =
            vc.channel2 =
            vc.channel3 = strtol(p, &p, 10);
        } else {
            vc.channel0 =
            vc.channel1 =
            vc.channel2 =
            vc.channel3 = 128;
        }
        printf("Set Volume vol=%d\n", vc.channel0);
        ret = ioctl(cd, CDROMVOLCTRL, &vc);
    } else if (match(p, "quit", 1))
        return -1;
    else {
        printf("Unknown cmd '%s'\n", p);
        return 1;
    }
    if (ret == -1) {
        perror(line);
        return 1;
    } else if (!isatty(0))
        printf("Doing '%s'\n", line);
   return 0;
}

int
main(int argc, char **argv) {
    char line[256];

    if ((cd = open("/dev/cd0", O_RDONLY)) == -1) {
        perror("open");
        return 1;
    }
    if (argc == 1)
        while (fgets(line, sizeof line, stdin) && parse(line) != -1);
    else
        while (*++argv && parse(*argv) != -1);
    close(cd);
    return 0;
}
