/*
 * This will be for all generic SCSI commands and stuff
 */

#ifndef _SCSI_H_INCLUDED
#define _SCSI_H_INCLUDED

typedef unsigned long be_32;
typedef unsigned short be_16;
typedef struct {
	unsigned char x[3];
} be_24;

/*
 *	SCSI Command Code definitions
 *
 *	The first byte of the CDB is the operation code.  The top 3 bits
 *	are the group code field and the bottom 5 bits are the command code
 *	field.
 *
 *	Group 0 - six-byte commands
 *	Group 1 - ten-byte commands
 *	Group 2 - ten-byte commands
 *	Group 3 - reserved
 *	Group 4 - reserved
 *	Group 5 - twelve-byte commands
 *	Group 6 - vendor specific
 *	Group 7 - vendor specific
 */

#define TARG_MAX	8		/*	Max number of SCSI targets */
#define LUN_MAX		8		/*	Max number of LUNs per target */

#define M_GROUP(cmd)	((cmd)>>5)
#define	SCSI_OP(grp_code,cmd_code)	(((grp_code)<<5)|((cmd_code)&0x1f))


/*
 *	Group 0 Command Codes (00H - 1FH)
 *	(six byte commands)
 */
#define SC_UNIT_RDY		0x00	/*	Test Unit Ready */
#define SC_REZERO		0x01	/*	Rezero Unit */
#define SC_REWIND		0x01	/*	Rewind (sequential-access device) */
#define SC_VEN_02		0x02	/*	Vendor specific */
#define SC_RQ_SENSE		0x03	/*	Request Sense */
#define SC_FORMAT		0x04	/*	Format unit */
#define SC_LIMITS		0x05	/*	Read Block Limits (sequential-access device) */
#define SC_VEN_06		0x06	/*	Vendor specific */
#define SC_REASSIGN		0x07	/*	Reassign Blocks */
#define SC_READ			0x08	/*	Read */
#define SC_VEN_09		0x09	/*	Vendor specific */
#define SC_WRITE		0x0a	/*	Write */
#define SC_SEEK			0x0b	/*	Seek */
#define SC_TRACK_SEL	0x0b	/*	Track Select (sequential-access device) */
#define SC_VEN_0C		0x0C	/*	Vendor specific */
#define SC_VEN_0D		0x0D	/*	Vendor specific */
#define SC_VEN_0E		0x0E	/*	Vendor specific */
#define SC_RD_REVERSE	0x0f	/*	Read Reverse (sequential-access device) */
#define SC_WR_FM		0x10	/*	Write Filemarks (sequential-access device) */
#define SC_SPACE		0x11	/*	Space (sequential-access device) */
#define SC_INQUIRY		0x12	/*	Inquiry */
#define SC_VERIFY		0x13	/*	Verify */
#define SC_RECOVER_BD	0x14	/*	Recover Buffered Data (sequential-access device) */	
#define SC_MSELECT		0x15	/*	Mode Select */
#define SC_RES_UNIT		0x16	/*	Reserve Unit */
#define SC_REL_UNIT		0x17	/*	Release Unit */
#define SC_COPY			0x18	/*	Copy */
#define SC_ERASE		0x19	/*	Erase (sequential-access device) */
#define SC_MSENSE		0x1a	/*	Mode Sense */
#define SC_SPINDLE		0x1b	/*	Start/Stop Unit */
#define SC_LOAD			0x1b	/*	Load/Unload (sequential-access device) */
#define SC_RCV_DIAG		0x1c	/*	Receive Diagnostic Results */
#define SC_SND_DIAG		0x1d	/*	Send Diagnostic */
#define SC_REMOVAL		0x1e	/*	Prevent/Allow Medium Removal */		
#define SC_RES_1F		0x1f	/*	(reserved) */


/*
 *	Group 1 Command Codes (20H - 3FH)
 *	(ten byte commands)
 */
#define SC1_VEN_20		0x20	/*	Vendor specific */
#define SC1_VEN_21		0x21	/*	Vendor specific */
#define SC1_VEN_22		0x22	/*	Vendor specific */
#define SC1_VEN_23		0x23	/*	Vendor specific */
#define SC1_SET_WIN		0x24	/*	Set Window (scanner device) */
#define SC1_GET_WIN		0x25	/*	Get Window (scanner device) */
#define SC1_RD_CAP		0x25	/*	Read Capacity */
#define SC1_VEN_26		0x26	/*	Vendor specific */
#define SC1_VEN_27		0x27	/*	Vendor specific */
#define SC1_READ		0x28	/*	Read */
#define SC1_RD_GEN		0x29	/*	Read Generation (optical-memory device) */
#define SC1_WRITE		0x2a	/*	Write */
#define SC1_SEEK		0x2b	/*	Seek */
#define SC1_LOCATE		0x2b	/*	Locate (sequential-access) */
#define SC1_ERASE		0x2c	/*	Erase (optical-memory) */
#define SC1_RD_UB		0x2d	/*	Read Updated Block (optical-memory) */
#define SC1_WR_VER		0x2e	/*	Write with Verify */
#define SC1_VERIFY		0x2f	/*	Verify */
#define SC1_SRCH_HI		0x30	/*	Search Data High */
#define SC1_SRCH_EQ		0x31	/*	Search Data Equal */
#define SC1_SRCH_LO		0x32	/*	Search Data Low */
#define SC1_LIMITS		0x33	/*	Set Limits */
#define SC1_PREFETCH	0x34	/*	Pre-fetch (pre-read into cache) */
#define SC1_RD_POS		0x34	/*	Read position (sequential-access device) */
#define SC1_SYNC		0x35	/*	Synchronize Cache */
#define SC1_CACHE		0x36	/*	Loca/Unlock Cache */
#define SC1_RD_DEF		0x37	/*	Read Defect Data */
#define SC1_SCAN		0x38	/*	Medium Scan (optical-memory) */
#define SC1_COMPARE		0x39	/*	Compare */
#define SC1_CPY_VER		0x3a	/*	Copy and Verify */
#define SC1_WR_BUF		0x3b	/*	Write Buffer */
#define SC1_RD_BUF		0x3c	/*	Read Buffer */
#define SC1_UPD_BLK		0x3d	/*	Update Block (optical-memory) */
#define SC1_RD_LONG		0x3e	/*	Read long */
#define SC1_WR_LONG		0x3f	/*	Write long */

/*
 *	Group 2 Command Codes (40H - 5FH)
 *	(ten byte commands)
 */
#define SC2_CHG_DEF		0x40	/*	Change definition (SCSI-1, SCSI-2, CCS) */
#define SC2_WR_SAME		0x41	/*	Write same */
#define SC2_RD_SUBCH	0x42	/*	Read Sub-Channel (cd-rom) */
#define SC2_RD_TOC		0x43	/*	Read TOC (cd-rom) */
#define SC2_RD_HDR		0x44	/*	Read Header (cd-rom) */
#define SC2_PLAY		0x45	/*	Play Audio(10) (cd-rom) */
#define SC2_RES_46		0x46	/*	(reserved) */
#define SC2_PLAY_MSF	0x47	/*	Play Audio MSF (cd-rom) */
#define SC2_PLAY_TI		0x48	/*	Play Audio Track Index (cd-rom) */
#define SC2_PLAY_TR		0x49	/*	Play Track Relative(10) (cd-rom) */
#define SC2_RES_4A		0x4a	/*	(reserved) */
#define SC2_PAUSE		0x4b	/*	Pause Resume (cd-rom) */
#define SC2_LSELECT		0x4c	/*	Log Select */
#define SC2_LSENSE		0x4d	/*	Log Sense */
#define SC2_RES_4E		0x4e	/*	(reserved) */
#define SC2_RES_4F		0x4f	/*	(reserved) */
#define SC2_MSELECT		0x55	/*	Mode Select */
#define SC2_MSENSE		0x5a	/*	Mode Sense */

/*
 *	Group 3 and 4, command codes 60H-9FH are reserved
 */

/*
 *	Group 5 command codes (A0H - BFH)
 *	(twelve byte commands)
 */
#define SC5_RES_A0		0xa0	/*	(reserved) */
#define SC5_RES_A1		0xa1	/*	(reserved) */
#define SC5_RES_A2		0xa2	/*	(reserved) */
#define SC5_RES_A3		0xa3	/*	(reserved) */
#define SC5_RES_A4		0xa4	/*	(reserved) */
#define SC5_MOVE		0xa5	/*	Move Medium (media-changer) */
#define SC5_PLAY		0xa5	/*	Play Audio (cd-rom) */
#define SC5_EXCHANGE	0xa6	/*	Exchange Medium (media-changer) */
#define SC5_RES_A7		0xa7	/*	(reserved) */
#define SC5_GET_MSG		0xa8	/*	Get Message (communications device) */
#define SC5_READ		0xa8	/*	Read (cd-rom) */
#define SC5_PLAY_TR		0xa9	/*	Play Track Relative (cd-rom) */
#define SC5_SEND_MSG	0xaa	/*	Send Message (communications device) */
#define SC5_WRITE		0xaa	/*	Write (optical-memory) */
#define SC5_RES_AB		0xab	/*	(reserved) */
#define SC5_ERASE		0xac	/*	Erase (optical-memory) */
#define SC5_RES_AD		0xad	/*	(reserved) */
#define SC5_WR_VER		0xae	/*	Write and Verify (optical-memory) */
#define SC5_VERIFY		0xaf	/*	Verify (optical-memory) */
#define SC5_SRCH_HI		0xb0	/*	Search Data High (cd-rom) */
#define SC5_SRCH_EQ		0xb1	/*	Search Data Equal (cd-rom) */
#define SC5_SRCH_LO		0xb2	/*	Search Data Low (cd-rom) */
#define SC5_LIMITS		0xb3	/*	Set Limits (cd-rom) */

/*
 *	Group 6 and 7, command codes C0H-FFH are vendor specific
 */
/*
 *	(Some WORMs)
 */
#define SC6_STANDBY		0xce	/*	Standby Check */
#define SC6_LOAD		0xcf	/*	Load/Unload Unit */
#define SC6_RWD_DCB		0xd2	/*	Read DCB */

#define SC7_DREAD		0xee	/*	Disk Read */
#define SC7_DWRITE		0xef	/*	Disk Write */

/*
 *	SCSI sub-command codes
 */
#define SPACE_BLOCKS	0x00	/*	Space to Nth block */
#define SPACE_FM		0x01	/*	Space to Nth filemark */
#define SPACE_SEQ_FM	0x02	/*	Space to first sequence of N filemarks */
#define SPACE_PHYS_EOD	0x03	/*	Space to physical end-of-data */

/*
 *	Other masks.
 */
#define SC_G0_BLK_LIMIT	0xffe00000L	/*	Group 0 read/write can only handle a 21bit block address */

/*
 *	SCSI status bytes
 *
 *	(as per ANSI X3.131-1986 -- Table 14-2)
 *
 *	The top most bit is reserved and bits 6, 5 and 0 are vendor
 *  unique, hence a mask may be necessary.
 */
#define SCS_MASK		0x1e	/*	Mask valid bits */
#define SCS_MASKR		0x80	/*	Mask reserved bit */
#define SCS_MASKV		0x61	/*	Mask vendor unique bits */

#define SCS_GOOD		0x00	/*	Good */
#define SCS_CHECK		0x02	/*	Check Condition */
#define SCS_MET			0x04	/*	Condition Met/Good */
#define SCS_BUSY		0x08	/*	Target/LUN is Busy */
#define SCS_INTER		0x10	/*	Intermediate/Good */
#define SCS_CONFLICT	0x30	/*	Reservation Conflict */

#define SCSI_I_M  (SCS_INTER|SCS_MET)	/*	Intermediate/Condition Met/Good */


/*
 *	Command Descriptor Block structure definitions
 */
struct _group0 {
	char	g0_opcode;		/*	Operation Code */
	be_24	g0_block;		/*	Block address (top 3 bits contain LUN) */
	char	g0_datalen;		/*	Data length (xfer|params|alloc) */
	char	g0_control;		/*	Control char */
};

#define LEN_G0	sizeof(struct _group0)


/*
 *	Group 1 and 2 - ten-byte
 */
struct _group1 {
	char	g1_opcode;		/*	Operation Code */
	char	g1_lun;			/*	Logical Unit Number */
	be_32	g1_block;		/*	Block address */
	char	g1_reserved;
	be_16	g1_datalen;		/*	Data length (xfer|params|alloc) */
	char	g1_control;		/*	Control char */
};
#define LEN_G1	sizeof(struct _group1)

/*
 *	Group 5 - twelve-byte
 */
struct _group5 {
	char	g5_opcode;		/*	Operation Code */
	char	g5_lun;			/*	Logical Unit Number */
	be_32	g5_block;		/*	Block address */
	be_32	g5_datalen;		/*	Data length (xfer|params|alloc) */
	char	g5_reserved;
	char	g5_control;		/*	Control char */
};
#define LEN_G5	sizeof(struct _group5)

/*
 *	CDB Flags
 */
#define CF_LINK			0x01	/*	Linked-command indication */
#define CF_FLAG			0x02	/*	Linked-command with flag bit */
#define CF_VENDOR0		0x40	/*	Vendor unique bits */
#define CF_VENDOR1		0x80

#define CF_FUA			0x08
#define CF_DPO			0x10


/*
 *	Generic Group
 */
struct _groupg {
	char	gg_char0;
	char	gg_char1;
	char	gg_char2;
	char	gg_char3;
	char	gg_char4;
	char	gg_char5;
	char	gg_char6;
	char	gg_char7;
	char	gg_char8;
	char	gg_char9;
	char	gg_char10;
	char	gg_char11;
};


/*
 *	INQUIRY Data flags
 */
#define ID_RMB		0x80	/*	Removable Medium Bit */
#define ID_AENC		0x80	/*	Asynchronous Event Notificaton Capability */
#define ID_TRM_IOP	0x40	/*	Terminate I/O Process bit */
#define ID_RDF		0x0f	/*	Response Data Format mask */
#define ID_REL_ADR	0x80	/*	Relative Addressing bit */
#define ID_WBUS32	0x40	/*	Wide Bus 32 (32 bit bus) support */
#define ID_WBUS16	0x20	/*	Wide Bus 16 (16 bit bus) support */
#define ID_SYNC		0x10	/*	Synchronous Data Transfers supported */
#define ID_LINKED	0x08	/*	Linked commands supported */
#define ID_RES		0x04	/*	(reserved) */
#define ID_CMD_QUE	0x02	/*	Tagged Command Queuing supported */
#define ID_SFT_RST	0x01	/*	Soft RESET alternative supported */

#define ID_QUAL_MASK		0xe0
#define ID_QUAL_AVAIL		0x00
#define ID_QUAL_NOTAVAIL	0x20
#define ID_QUAL_RESERVED	0x40
#define ID_QUAL_NOTSUP		0x60
#define ID_QUAL_VENDOR		0x80

/*
 *  Definition of the SCSI-2 INQUIRY data format
 *	(only the first 3 chars are valid for SCSI-1)
 */
typedef struct _scsi_inquiry {
	char	id_peripheral;	/*	Peripheral Qualifier/Device Type */
	char	id_rmb;			/*	Removable Medium Bit */
	char	id_ver;			/*	Versions (ISO, ECMA, ANSI) */
	char	id_rdf;			/*	Flags and Response Data Format */
	char	id_adlen;		/*	Additional Data Length */
	char	id_res1;
	char	id_res2;
	char	id_flags;		/*	Flags */
	char	id_vend_id[8];	/*	Vendor identification */
	char	id_prod_id[16];	/*	Product identification */
	char	id_prod_rev[4];	/*	Product revision level */
	char	id_vend_spc[20];/*	Vendor Specific */
	char	id_res3[40];
} SCSI_INQUIRY;

/*
 *	Sense Keys (structure "sense", field "sd_sense")
 */
#define SK_MASK			0x0F	/*	mask to sd_sense field for key */
#define SK_NO_SENSE		0		/*	No sense data (no error) */
#define SK_RECOVERED	1		/*	Recovered error */
#define SK_NOT_RDY		2		/*	Device not ready */
#define SK_MEDIUM		3		/*	Medium error */
#define SK_HARDWARE		4		/*	Hardware error */
#define SK_ILLEGAL		5		/*	Illegal Request (bad command) */
#define SK_UNIT_ATN		6		/*	Unit Attention */
#define SK_DATA_PROT	7		/*	Data Protect */
#define SK_BLNK_CHK		8		/*	Blank Check */
#define SK_VENDOR		9		/*	Vendor Specific */
#define SK_CPY_ABORT	10		/*	Copy Aborted */
#define SK_CMD_ABORT	11		/*	Aborted Command */
#define SK_EQUAL		12		/*	Equal */
#define SK_VOL_OFL		13		/*	Volume Overflow */
#define SK_MISCMP		14		/*	Miscompare */
#define SK_RESERVED		15		/*	Reserved */

/*
 *	Definition of the SCSI-2 Sense Data Structure
 *	(similar to SCSI-1 extended sense data)
 *
 *	Note: The field "sd_err" has the following format:
 *        bit    7	- Address valid bit
 *		  bits 6-4  - Error class
 *		  bits 3-0  - Error code 
 *
 *	Error classes 0-6 are vendor unique and also indicate that the
 *  sense data is in _nonextended_ format. (i.e. not usually used)
 *		struct _scsi_nonextended_sense {
 *			char	sd_err;
 *			be_32	sd_block_address;
 *		};
 *
 *	An error class of 7 and an error code of 0 (70H) indicate SCSI-1
 *	extended sense data format (or SCSI-2 sense data format).
 *
 *	An error class of 7 and an error code of 1 (71H) indicate SCSI-2
 *	deferred errors.
 *
 *	Error codes 72H to 7EH are reserved and error code 7FH indicates
 *	a vendor-specific sense data format.
 *
 */
typedef struct _scsi_sense {
	char	sd_err;			/*	Info Valid flag, Error Class and Error Code */
	char	sd_seg;			/*	Segment number */
	char	sd_sense;		/*	Sense key & Flags */
	be_32	sd_info;		/*	Information (32bit big-endian value) */
	char	sd_asl;			/*	Additional Sense Length */
	be_32	sd_csinfo;		/*	Command-Specific Information */
	char	sd_asc;			/*	Additional Sense Code */
	char	sd_ascq;		/*	Additional Sense Code Qualifier */
	char	sd_fruc;		/*	Field Replaceable Unit Code */
	char	sd_sks_flags;	/*	Sense Key Specific Flags */
	be_16	sd_sks_data;	/*	Sense Key Specific Data (16bit big-endian) */
	char	sd_asb[2];		/*	Additional Sense chars (Max 256-18) */
} SCSI_SENSE;

/*
 *	SCSI Device Type definitions
 *	(types 0Ah - 1Eh are reserved)
 */
#define D_DIR_ACC		0		/*	Direct Access device */
#define D_SEQ_ACC		1		/*	Sequential Access device */
#define D_PRINTER		2		/*	Printer device */
#define D_PROCESSOR		3		/*	Processor device */
#define D_WORM			4		/*	Write Once Read Multiple device */
#define D_READ_ONLY		5		/*	Read Only (CD-ROM) device */
#define D_SCANNER		6		/*	Scanner device */
#define D_OPTICAL		7		/*	Optical Memory device */
#define D_MEDIA_CHG		8		/*	Media Changer device */
#define D_COMM			9		/*	Communication device */
#define D_UNKNOWN		0x1f	/*	Unknown device type */
#define D_MASK			0x1f	/*	Mask for device type */

/*
 *	Mode Page Codes
 */
#define MP_RW_ERR		0x01	/*	Read-Write error recovery page */
#define MP_CONNECT		0x02	/*	Disconnect-reconnect page */
#define MP_FMT_DEV		0x03	/*	Format device page */
#define MP_GEOMETRY		0x04	/*	Rigid disk geometry page */
#define MP_FLEX_DSK		0x05	/*	Flexible disk page */
#define MP_RESERVED		0x06	/*	(reserved) */
#define MP_VFY_ERR		0x07	/*	Verify error recovery page */
#define MP_CACHING		0x08	/*	Caching page */
#define MP_PERIPHERAL	0x09	/*	Peripheral device page */
#define MP_CONTROL		0x0a	/*	Control mode page */
#define MP_MEDIUM		0x0b	/*	Medium types supported page */
#define MP_PARTITION	0x0c	/*	Notch and Partition page */
#define MP_ALL			0x3f	/*	Return all pages ("mode sense" only) */


/*
 *	Definition of driver's Target Descriptor Block
 */
typedef struct _tdb {
	unsigned	td_id;			/*	Target ID */
	unsigned	td_luns;		/*	LUNs */
	unsigned	td_flags;		/*	Flags */
	char		td_version;		/*	SCSI Version */
	char		td_devtype;		/*	Device Type */
	char		td_fblk_dblk;	/*	Number of fsys blocks per device block */
	char		td_shift;		/*	Shift factor for blocks > 512 chars */
	char		td_mask;		/*	Mask used for blocks > 512 chars */
	char		td_spare;
	long		td_numblocks;	/*	Number of blocks */
	unsigned	td_blksize;		/*	Block size (in bytes) */
	unsigned	td_blks_cyl;	/*	Blocks per cylinder */
	unsigned	td_err_sense;	/*	Error Sense Key */
	unsigned	td_err_asc;		/*	Additional Sense Code and Qualifier (hi/lo) */
	long		td_err_blk;		/*	Block number of error (if applicable) */
	unsigned	td_spare2[2];
} SCSI_TDB;

/*
 *	Mode sense parameter header (6)
 */
typedef struct _msense_header_6 {
	char	sense_len;
	char	medium_type;
	char	device_specific;
	char	block_desc_length;
} MSENSE_HEADER_6;

/*
 *	Mode sense parameter header (10)
 */
typedef struct _msense_header_10 {
	be_16	sense_len;
	char	medium_type;
	char	device_specific;
	char	zero1;
	char	zero2;
	be_16	block_desc_length;
} MSENSE_HEADER_10;

/*
 *	Mode sense block descriptor
 */
typedef struct _msense_block_descriptor {
	char	density;
	be_24	num_blocks;
	char	zero1;
	be_24	block_len;
} MSENSE_BLOCK_DESCRIPTOR;

#endif
