const char *sense_keys[] = {
	"NO_SENSE",
	"RECOVERED_ERROR",
	"NOT_READY",
	"MEDIUM_ERROR",
	"HARDWARE_ERROR",
	"ILLEGAL_REQUEST",
	"UNIT_ATTENTION",
	"DATA_PROTECT",
	"BLANK_CHECK",
	"vendor_specific",
	"COPY_ABORTED",
	"ABORTED_COMMAND",
	"EQUAL",
	"VOLUME_OVERFLOW",
	"MISCOMPARE",
	"RESERVED"
};


/*
 * SCSI-3 ASC and ASCQ assignments as of April 6, 1993
 * 
 * DIRECT ACCESS DEVICE                0x0001
 * SEQUENTIAL ACCESS DEVICE            0x0002
 * PRINTER DEVICE                      0x0004
 * PROCESSOR DEVICE                    0x0008
 * WRITE ONCE READ MULTIPLE DEVICE     0x0010
 * READ ONLY (CD-ROM) DEVICE           0x0020
 * SCANNER DEVICE                      0x0040
 * OPTICAL MEMORY DEVICE               0x0080
 * MEDIA CHANGER DEVICE                0x0100
 * COMMUNICATION DEVICE                0x0200
 *
 * If either asc or ascq are >= 0x80, the code is
 * vendor specific
 */
const struct scsi_asc {
	unsigned char	asc;	/* Additional Sense Code */
	signed char		ascq;	/* Additional Sense Code Qualifier */
							/* a -1 indicates part of message */
	unsigned		type;	/* Device types */
	const char		*desc;	/* ASC description */
} scsi_asc[] = {
	{ 0x00, 0x00, 0x03ff, "NO ADDITIONAL SENSE INFORMATION" },
	{ 0x00, 0x01, 0x0100, "FILEMARK DETECTED" },
	{ 0x00, 0x02, 0x0108, "END-OF-PARTITION/MEDIUM DETECTED" },
	{ 0x00, 0x03, 0x0100, "SETMARK DETECTED" },
	{ 0x00, 0x04, 0x0108, "BEGINNING-OF-PARTITION/MEDIUM DETECTED" },
	{ 0x00, 0x05, 0x0108, "END-OF-DATA DETECTED" },
	{ 0x00, 0x06, 0x03ff, "I/O PROCESS TERMINATED" },
	{ 0x00, 0x11, 0x0010, "AUDIO PLAY OPERATION IN PROGRESS" },
	{ 0x00, 0x12, 0x0010, "AUDIO PLAY OPERATION PAUSED" },
	{ 0x00, 0x13, 0x0010, "AUDIO PLAY OPERATION SUCCESSFULLY COMPLETED" },
	{ 0x00, 0x14, 0x0010, "AUDIO PLAY OPERATION STOPPED DUE TO ERROR" },
	{ 0x00, 0x15, 0x0010, "NO CURRENT AUDIO STATUS TO RETURN" },
	{ 0x01, 0x00, 0x0224, "NO INDEX/SECTOR SIGNAL" },
	{ 0x02, 0x00, 0x0236, "NO SEEK COMPLETE" },
	{ 0x03, 0x00, 0x03ac, "PERIPHERAL DEVICE WRITE FAULT" },
	{ 0x03, 0x01, 0x0100, "NO WRITE CURRENT" },
	{ 0x03, 0x02, 0x0100, "EXCESSIVE WRITE ERRORS" },
	{ 0x04, 0x00, 0x03ff, "LOGICAL UNIT NOT READY" },
	{ 0x04, 0x01, 0x03ff, "LOGICAL UNIT IS IN PROCESS OF BECOMING READY" },
	{ 0x04, 0x02, 0x03ff, "LOGICAL UNIT NOT READY" },
	{ 0x04, 0x03, 0x03ff, "LOGICAL UNIT NOT READY" },
	{ 0x04, 0x04, 0x0384, "LOGICAL UNIT NOT READY" },
	{ 0x05, 0x00, 0x03bf, "LOGICAL UNIT DOES NOT RESPOND TO SELECTION" },
	{ 0x06, 0x00, 0x0236, "NO REFERENCE POSITION FOUND" },
	{ 0x07, 0x00, 0x03be, "MULTIPLE PERIPHERAL DEVICES SELECTED" },
	{ 0x08, 0x00, 0x03bf, "LOGICAL UNIT COMMUNICATION FAILURE" },
	{ 0x08, 0x01, 0x03bf, "LOGICAL UNIT COMMUNICATION TIME-OUT" },
	{ 0x08, 0x02, 0x03bf, "LOGICAL UNIT COMMUNICATION PARITY ERROR" },
	{ 0x09, 0x00, 0x0334, "TRACK FOLLOWING ERROR" },
	{ 0x09, 0x01, 0x0034, "TRACKING SERVO FAILURE" },
	{ 0x09, 0x02, 0x0034, "FOCUS SERVO FAILURE" },
	{ 0x09, 0x03, 0x0034, "SPINDLE SERVO FAILURE" },
	{ 0x09, 0x04, 0x0334, "HEAD SELECT FAULT" },
	{ 0x0a, 0x00, 0x03ff, "ERROR LOG OVERFLOW" },
	{ 0x0c, 0x00, 0x0108, "WRITE ERROR" },
	{ 0x0c, 0x02, 0x0224, "WRITE ERROR - AUTO REALLOCATION FAILED" },
	{ 0x0c, 0x03, 0x0224, "WRITE ERROR - RECOMMEND REASSIGNMENT" },
	{ 0x0c, 0x04, 0x0324, "COMPRESSION CHECK MISCOMPARE ERROR" },
	{ 0x0c, 0x05, 0x0324, "DATA EXPANSION OCCURRED DURING COMPRESSION" },
	{ 0x0c, 0x06, 0x0324, "BLOCK NOT COMPRESSABLE" },
	{ 0x10, 0x00, 0x0224, "ID CRC OR ECC ERROR" },
	{ 0x11, 0x00, 0x033c, "UNRECOVERED READ ERROR" },
	{ 0x11, 0x01, 0x032c, "READ RETRIES EXHAUSTED" },
	{ 0x11, 0x02, 0x032c, "ERROR TOO LONG TO CORRECT" },
	{ 0x11, 0x03, 0x032c, "MULTIPLE READ ERRORS" },
	{ 0x11, 0x04, 0x0224, "UNRECOVERED READ ERROR - AUTO REALLOCATE FAILED" },
	{ 0x11, 0x05, 0x0034, "L-EC UNCORRECTABLE ERROR" },
	{ 0x11, 0x06, 0x0034, "CIRC UNRECOVERED ERROR" },
	{ 0x11, 0x07, 0x0024, "DATA RE-SYNCHRONIZATION ERROR" },
	{ 0x11, 0x08, 0x0100, "INCOMPLETE BLOCK READ" },
	{ 0x11, 0x09, 0x0100, "NO GAP FOUND" },
	{ 0x11, 0x0a, 0x0304, "MISCORRECTED ERROR" },
	{ 0x11, 0x0b, 0x0224, "UNRECOVERED READ ERROR - RECOMMEND REASSIGNMENT" },
	{ 0x11, 0x0c, 0x0224, "UNRECOVERED READ ERROR - RECOMMEND REWRITE THE DATA" },
	{ 0x11, 0x0d, 0x0334, "DE-COMPRESSION CRC ERROR" },
	{ 0x11, 0x0e, 0x0334, "CANNOT DECOMPRESS USING DECLARED ALGORITHM" },
	{ 0x12, 0x00, 0x0224, "ADDRESS MARK NOT FOUND FOR ID FIELD" },
	{ 0x13, 0x00, 0x0224, "ADDRESS MARK NOT FOUND FOR DATA FIELD" },
	{ 0x14, 0x00, 0x03bc, "RECORDED ENTITY NOT FOUND" },
	{ 0x14, 0x01, 0x0334, "RECORD NOT FOUND" },
	{ 0x14, 0x02, 0x0100, "FILEMARK OR SETMARK NOT FOUND" },
	{ 0x14, 0x03, 0x0100, "END-OF-DATA NOT FOUND" },
	{ 0x14, 0x04, 0x0100, "BLOCK SEQUENCE ERROR" },
	{ 0x14, 0x05, 0x0324, "RECORD NOT FOUND - RECOMMEND REASSIGNMENT" },
	{ 0x14, 0x06, 0x0324, "RECORD NOT FOUND - DATA AUTO-REALLOCATED" },
	{ 0x15, 0x00, 0x03be, "RANDOM POSITIONING ERROR" },
	{ 0x15, 0x01, 0x03be, "MECHANICAL POSITIONING ERROR" },
	{ 0x15, 0x02, 0x0334, "POSITIONING ERROR DETECTED BY READ OF MEDIUM" },
	{ 0x16, 0x00, 0x0224, "DATA SYNCHRONIZATION MARK ERROR" },
	{ 0x16, 0x01, 0x0224, "DATA SYNC ERROR - DATA REWRITTEN" },
	{ 0x16, 0x02, 0x0224, "DATA SYNC ERROR - RECOMMEND REWRITE" },
	{ 0x16, 0x03, 0x0224, "DATA SYNC ERROR - DATA AUTO-REALLOCATED" },
	{ 0x16, 0x04, 0x0224, "DATA SYNC ERROR - RECOMMEND REASSIGNMENT" },
	{ 0x17, 0x00, 0x033c, "RECOVERED DATA WITH NO ERROR CORRECTION APPLIED" },
	{ 0x17, 0x01, 0x033c, "RECOVERED DATA WITH RETRIES" },
	{ 0x17, 0x02, 0x0334, "RECOVERED DATA WITH POSITIVE HEAD OFFSET" },
	{ 0x17, 0x03, 0x0334, "RECOVERED DATA WITH NEGATIVE HEAD OFFSET" },
	{ 0x17, 0x04, 0x0034, "RECOVERED DATA WITH RETRIES AND/OR CIRC APPLIED" },
	{ 0x17, 0x05, 0x0234, "RECOVERED DATA USING PREVIOUS SECTOR ID" },
	{ 0x17, 0x06, 0x0224, "RECOVERED DATA WITHOUT ECC - DATA AUTO-REALLOCATED" },
	{ 0x17, 0x07, 0x0224, "RECOVERED DATA WITHOUT ECC - RECOMMEND REASSIGNMENT" },
	{ 0x17, 0x08, 0x0224, "RECOVERED DATA WITHOUT ECC - RECOMMEND REWRITE" },
	{ 0x17, 0x09, 0x0224, "RECOVERED DATA WITHOUT ECC - DATA REWRITTEN" },
	{ 0x18, 0x00, 0x0334, "RECOVERED DATA WITH ERROR CORRECTION APPLIED" },
	{ 0x18, 0x01, 0x0234, "RECOVERED DATA WITH ERROR CORRECTION & RETRIES APPLIED" },
	{ 0x18, 0x02, 0x0234, "RECOVERED DATA - DATA AUTO-REALLOCATED" },
	{ 0x18, 0x03, 0x0010, "RECOVERED DATA WITH CIRC" },
	{ 0x18, 0x04, 0x0010, "RECOVERED DATA WITH L-EC" },
	{ 0x18, 0x05, 0x0234, "RECOVERED DATA - RECOMMEND REASSIGNMENT" },
	{ 0x18, 0x06, 0x0234, "RECOVERED DATA - RECOMMEND REWRITE" },
	{ 0x18, 0x07, 0x0224, "RECOVERED DATA WITH ECC - DATA REWRITTEN" },
	{ 0x19, 0x00, 0x0204, "DEFECT LIST ERROR" },
	{ 0x19, 0x01, 0x0204, "DEFECT LIST NOT AVAILABLE" },
	{ 0x19, 0x02, 0x0204, "DEFECT LIST ERROR IN PRIMARY LIST" },
	{ 0x19, 0x03, 0x0204, "DEFECT LIST ERROR IN GROWN LIST" },
	{ 0x1a, 0x00, 0x03ff, "PARAMETER LIST LENGTH ERROR" },
	{ 0x1b, 0x00, 0x03ff, "SYNCHRONOUS DATA TRANSFER ERROR" },
	{ 0x1c, 0x00, 0x0204, "DEFECT LIST NOT FOUND" },
	{ 0x1c, 0x01, 0x0204, "PRIMARY DEFECT LIST NOT FOUND" },
	{ 0x1c, 0x02, 0x0204, "GROWN DEFECT LIST NOT FOUND" },
	{ 0x1d, 0x00, 0x0224, "MISCOMPARE DURING VERIFY OPERATION" },
	{ 0x1e, 0x00, 0x0224, "RECOVERED ID WITH ECC CORRECTION" },
	{ 0x1f, 0x00, 0x0204, "PARTIAL DEFECT LIST TRANSFER" },
	{ 0x20, 0x00, 0x03ff, "INVALID COMMAND OPERATION CODE" },
	{ 0x21, 0x00, 0x0336, "LOGICAL BLOCK ADDRESS OUT OF RANGE" },
	{ 0x21, 0x01, 0x0336, "INVALID ELEMENT ADDRESS" },
	{ 0x22, 0x00, 0x0200, "ILLEGAL FUNCTION (SHOULD USE 20 00" },
	{ 0x24, 0x00, 0x03ff, "INVALID FIELD IN CDB" },
	{ 0x25, 0x00, 0x03ff, "LOGICAL UNIT NOT SUPPORTED" },
	{ 0x26, 0x00, 0x03ff, "INVALID FIELD IN PARAMETER LIST" },
	{ 0x26, 0x01, 0x03ff, "PARAMETER NOT SUPPORTED" },
	{ 0x26, 0x02, 0x03ff, "PARAMETER VALUE INVALID" },
	{ 0x26, 0x03, 0x03ff, "THRESHOLD PARAMETERS NOT SUPPORTED" },
	{ 0x27, 0x00, 0x0324, "WRITE PROTECTED" },
	{ 0x28, 0x00, 0x03ff, "NOT READY TO READY TRANSITION" },
	{ 0x28, 0x01, 0x0336, "IMPORT OR EXPORT ELEMENT ACCESSED" },
	{ 0x29, 0x00, 0x03ff, "POWER ON" },
	{ 0x29, 0x01, 0x03ff, "POWER ON OCCURRED" },
	{ 0x29, 0x02, 0x03ff, "SCSI BUS RESET OCCURRED" },
	{ 0x29, 0x03, 0x03ff, "BUS DEVICE RESET MESSAGE OCCURRED" },
	{ 0x2a, 0x00, 0x03bf, "PARAMETERS CHANGED" },
	{ 0x2a, 0x01, 0x03bf, "MODE PARAMETERS CHANGED" },
	{ 0x2a, 0x02, 0x03bf, "LOG PARAMETERS CHANGED" },
	{ 0x2b, 0x00, 0x03fd, "COPY CANNOT EXECUTE SINCE HOST CANNOT DISCONNECT" },
	{ 0x2c, 0x00, 0x03ff, "COMMAND SEQUENCE ERROR" },
	{ 0x2c, 0x01, 0x0008, "TOO MANY WINDOWS SPECIFIED" },
	{ 0x2c, 0x02, 0x0008, "INVALID COMBINATION OF WINDOWS SPECIFIED" },
	{ 0x2d, 0x00, 0x0100, "OVERWRITE ERROR ON UPDATE IN PLACE" },
	{ 0x2f, 0x00, 0x03ff, "COMMANDS CLEARED BY ANOTHER INITIATOR" },
	{ 0x30, 0x00, 0x0336, "INCOMPATIBLE MEDIUM INSTALLED" },
	{ 0x30, 0x01, 0x0334, "CANNOT READ MEDIUM - UNKNOWN FORMAT" },
	{ 0x30, 0x02, 0x0334, "CANNOT READ MEDIUM - INCOMPATIBLE FORMAT" },
	{ 0x30, 0x03, 0x0300, "CLEANING CARTRIDGE INSTALLED" },
	{ 0x30, 0x04, 0x0324, "CANNOT WRITE MEDIUM - UNKNOWN FORMAT" },
	{ 0x30, 0x05, 0x0324, "CANNOT WRITE MEDIUM - INCOMPATIBLE FORMAT" },
	{ 0x30, 0x06, 0x0324, "CANNOT FORMAT MEDIUM - INCOMPATIBLE MEDIUM" },
	{ 0x31, 0x00, 0x0324, "MEDIUM FORMAT CORRUPTED" },
	{ 0x31, 0x01, 0x0284, "FORMAT COMMAND FAILED" },
	{ 0x32, 0x00, 0x0224, "NO DEFECT SPARE LOCATION AVAILABLE" },
	{ 0x32, 0x01, 0x0224, "DEFECT LIST UPDATE FAILURE" },
	{ 0x33, 0x00, 0x0100, "TAPE LENGTH ERROR" },
	{ 0x36, 0x00, 0x0080, "RIBBON" },
	{ 0x37, 0x00, 0x03bf, "ROUNDED PARAMETER" },
	{ 0x39, 0x00, 0x03bf, "SAVING PARAMETERS NOT SUPPORTED" },
	{ 0x3a, 0x00, 0x03be, "MEDIUM NOT PRESENT" },
	{ 0x3b, 0x00, 0x0180, "SEQUENTIAL POSITIONING ERROR" },
	{ 0x3b, 0x01, 0x0100, "TAPE POSITION ERROR AT BEGINNING-OF-MEDIUM" },
	{ 0x3b, 0x02, 0x0100, "TAPE POSITION ERROR AT END-OF-MEDIUM" },
	{ 0x3b, 0x03, 0x0080, "TAPE OR ELECTRONIC VERTICAL FORMS UNIT NOT READY" },
	{ 0x3b, 0x04, 0x0080, "SLEW FAILURE" },
	{ 0x3b, 0x05, 0x0080, "PAPER JAM" },
	{ 0x3b, 0x06, 0x0080, "FAILED TO SENSE TOP-OF-FORM" },
	{ 0x3b, 0x07, 0x0080, "FAILED TO SENSE BOTTOM-OF-FORM" },
	{ 0x3b, 0x08, 0x0100, "REPOSITION ERROR" },
	{ 0x3b, 0x09, 0x0008, "READ PAST END OF MEDIUM" },
	{ 0x3b, 0x0a, 0x0008, "READ PAST BEGINNING OF MEDIUM" },
	{ 0x3b, 0x0b, 0x0008, "POSITION PAST END OF MEDIUM" },
	{ 0x3b, 0x0c, 0x0008, "POSITION PAST BEGINNING OF MEDIUM" },
	{ 0x3b, 0x0d, 0x0336, "MEDIUM DESTINATION ELEMENT FULL" },
	{ 0x3b, 0x0e, 0x0336, "MEDIUM SOURCE ELEMENT EMPTY" },
	{ 0x3d, 0x00, 0x03ff, "INVALID BITS IN IDENTIFY MESSAGE" },
	{ 0x3e, 0x00, 0x03ff, "LOGICAL UNIT HAS NOT SELF-CONFIGURED YET" },
	{ 0x3f, 0x00, 0x03ff, "TARGET OPERATING CONDITIONS HAVE CHANGED" },
	{ 0x3f, 0x01, 0x03ff, "MICROCODE HAS BEEN CHANGED" },
	{ 0x3f, 0x02, 0x03ff, "CHANGED OPERATING DEFINITION" },
	{ 0x3f, 0x03, 0x03ff, "INQUIRY DATA HAS CHANGED" },
	{ 0x40, 0x00, 0x0200, "RAM FAILURE (SHOULD USE 40 NN)" },
	{ 0x40,   -1, 0x03ff, "DIAGNOSTIC FAILURE ON COMPONENT %02xH (80H-FFH)" },
	{ 0x41, 0x00, 0x0200, "DATA PATH FAILURE (SHOULD USE 40 NN)" },
	{ 0x42, 0x00, 0x0200, "POWER-ON OR SELF-TEST FAILURE (SHOULD USE 40 NN)" },
	{ 0x43, 0x00, 0x03ff, "MESSAGE ERROR" },
	{ 0x44, 0x00, 0x03ff, "INTERNAL TARGET FAILURE" },
	{ 0x45, 0x00, 0x03ff, "SELECT OR RESELECT FAILURE" },
	{ 0x46, 0x00, 0x03ff, "UNSUCCESSFUL SOFT RESET" },
	{ 0x47, 0x00, 0x03ff, "SCSI PARITY ERROR" },
	{ 0x48, 0x00, 0x03ff, "INITIATOR DETECTED ERROR MESSAGE RECEIVED" },
	{ 0x49, 0x00, 0x03ff, "INVALID MESSAGE ERROR" },
	{ 0x4a, 0x00, 0x03ff, "COMMAND PHASE ERROR" },
	{ 0x4b, 0x00, 0x03ff, "DATA PHASE ERROR" },
	{ 0x4c, 0x00, 0x03ff, "LOGICAL UNIT FAILED SELF-CONFIGURATION" },
	{ 0x4d,   -1, 0x03ff, "TAGGED OVERLAPPED COMMANDS (%d = QUEUE TAG)" },
	{ 0x4e, 0x00, 0x03ff, "OVERLAPPED COMMANDS ATTEMPTED" },
	{ 0x50, 0x00, 0x0100, "WRITE APPEND ERROR" },
	{ 0x50, 0x01, 0x0100, "WRITE APPEND POSITION ERROR" },
	{ 0x50, 0x02, 0x0100, "POSITION ERROR RELATED TO TIMING" },
	{ 0x51, 0x00, 0x0104, "ERASE FAILURE" },
	{ 0x52, 0x00, 0x0100, "CARTRIDGE FAULT" },
	{ 0x53, 0x00, 0x03be, "MEDIA LOAD OR EJECT FAILED" },
	{ 0x53, 0x01, 0x0100, "UNLOAD TAPE FAILURE" },
	{ 0x53, 0x02, 0x0336, "MEDIUM REMOVAL PREVENTED" },
	{ 0x54, 0x00, 0x0040, "SCSI TO HOST SYSTEM INTERFACE FAILURE" },
	{ 0x55, 0x00, 0x0040, "SYSTEM RESOURCE FAILURE" },
	{ 0x57, 0x00, 0x0010, "UNABLE TO RECOVER TABLE-OF-CONTENTS" },
	{ 0x58, 0x00, 0x0004, "GENERATION DOES NOT EXIST" },
	{ 0x59, 0x00, 0x0004, "UPDATED BLOCK READ" },
	{ 0x5a, 0x00, 0x03fe, "OPERATOR REQUEST OR STATE CHANGE INPUT (UNSPECIFIED)" },
	{ 0x5a, 0x01, 0x0336, "OPERATOR MEDIUM REMOVAL REQUEST" },
	{ 0x5a, 0x02, 0x0324, "OPERATOR SELECTED WRITE PROTECT" },
	{ 0x5a, 0x03, 0x0324, "OPERATOR SELECTED WRITE PERMIT" },
	{ 0x5b, 0x00, 0x03fe, "LOG EXCEPTION" },
	{ 0x5b, 0x01, 0x03fe, "THRESHOLD CONDITION MET" },
	{ 0x5b, 0x02, 0x03fe, "LOG COUNTER AT MAXIMUM" },
	{ 0x5b, 0x03, 0x03fe, "LOG LIST CODES EXHAUSTED" },
	{ 0x5c, 0x00, 0x0204, "RPL STATUS CHANGE" },
	{ 0x5c, 0x01, 0x0204, "SPINDLES SYNCHRONIZED" },
	{ 0x5c, 0x02, 0x0204, "SPINDLES NOT SYNCHRONIZED" },
	{ 0x5d, 0x00, 0x03ff, "FAILURE PREDICTION THRESHOLD EXCEEDED" },
	{ 0x5e, 0x00, 0x03fd, "LOW POWER CONDITION ACTIVE" },
	{ 0x5e, 0x01, 0x03fd, "IDLE CONDITION ACTIVATED BY TIMER" },
	{ 0x5e, 0x02, 0x03fd, "STANDBY CONDITION ACTIVATED BY TIMER" },
	{ 0x5e, 0x03, 0x03fd, "IDLE CONDITION ACTIVATED BY COMMAND" },
	{ 0x5e, 0x04, 0x03fd, "STANDBY CONDITION ACTIVATED BY COMMAND" },
	{ 0x60, 0x00, 0x0008, "LAMP FAILURE" },
	{ 0x61, 0x00, 0x0008, "VIDEO ACQUISITION ERROR" },
	{ 0x61, 0x01, 0x0008, "UNABLE TO ACQUIRE VIDEO" },
	{ 0x61, 0x02, 0x0008, "OUT OF FOCUS" },
	{ 0x62, 0x00, 0x0008, "SCAN HEAD POSITIONING ERROR" },
	{ 0x63, 0x00, 0x0010, "END OF USER AREA ENCOUNTERED ON THIS TRACK" },
	{ 0x64, 0x00, 0x0010, "ILLEGAL MODE FOR THIS TRACK" },
	{ 0x65, 0x00, 0x03ff, "VOLTAGE FAULT" },
	{ 0x70,   -1, 0x0100, "DECOMPRESSION EXCEPTION SHORT ALGORITHM ID OF %d" },
	{ 0x71, 0x00, 0x0100, "DECOMPRESSION EXCEPTION LONG ALGORITHM ID" },
};

const struct scsi_cmds {
	int				cmd;	/* Command code */
	unsigned		type;	/* Device types */
	const char		*desc;	/* Command description */
} scsi_cmds[] = {
	{ 0x00, 0x03ff, "Test Unit Ready" },
	{ 0x01, 0x03ff, "Rezero Unit" },
	{ 0x01, 0x03ff, "Rewind (sequential-access device)" },
	{ 0x02, 0x03ff, "Vendor specific" },
	{ 0x03, 0x03ff, "Request Sense" },
	{ 0x04, 0x03ff, "Format unit" },
	{ 0x05, 0x03ff, "Read Block Limits (sequential-access device)" },
	{ 0x06, 0x03ff, "Vendor specific" },
	{ 0x07, 0x03ff, "Reassign Blocks" },
	{ 0x08, 0x03ff, "Read" },
	{ 0x09, 0x03ff, "Vendor specific" },
	{ 0x0a, 0x03ff, "Write" },
	{ 0x0b, 0x03ff, "Seek" },
	{ 0x0b, 0x03ff, "Track Select (sequential-access device)" },
	{ 0x0C, 0x03ff, "Vendor specific" },
	{ 0x0D, 0x03ff, "Vendor specific" },
	{ 0x0E, 0x03ff, "Vendor specific" },
	{ 0x0f, 0x03ff, "Read Reverse (sequential-access device)" },
	{ 0x10, 0x03ff, "Write Filemarks (sequential-access device)" },
	{ 0x11, 0x03ff, "Space (sequential-access device)" },
	{ 0x12, 0x03ff, "Inquiry" },
	{ 0x13, 0x03ff, "Verify" },
	{ 0x14, 0x03ff, "Recover Buffered Data (sequential-access device)" },
	{ 0x15, 0x03ff, "Mode Select" },
	{ 0x16, 0x03ff, "Reserve Unit" },
	{ 0x17, 0x03ff, "Release Unit" },
	{ 0x18, 0x03ff, "Copy" },
	{ 0x19, 0x03ff, "Erase (sequential-access device)" },
	{ 0x1a, 0x03ff, "Mode Sense" },
	{ 0x1b, 0x03ff, "Start/Stop Unit" },
	{ 0x1b, 0x03ff, "Load/Unload (sequential-access device)" },
	{ 0x1c, 0x03ff, "Receive Diagnostic Results" },
	{ 0x1d, 0x03ff, "Send Diagnostic" },
	{ 0x1e, 0x03ff, "Prevent/Allow Medium Removal" },
	{ 0x1f, 0x03ff, "(reserved)" },
	{ 0x20, 0x03ff, "Vendor specific" },
	{ 0x21, 0x03ff, "Vendor specific" },
	{ 0x22, 0x03ff, "Vendor specific" },
	{ 0x23, 0x03ff, "Vendor specific" },
	{ 0x24, 0x03ff, "Set Window (scanner device)" },
	{ 0x25, 0x03ff, "Get Window (scanner device)" },
	{ 0x25, 0x03ff, "Read Capacity" },
	{ 0x26, 0x03ff, "Vendor specific" },
	{ 0x27, 0x03ff, "Vendor specific" },
	{ 0x28, 0x03ff, "Read" },
	{ 0x29, 0x03ff, "Read Generation (optical-memory device)" },
	{ 0x2a, 0x03ff, "Write" },
	{ 0x2b, 0x03ff, "Seek" },
	{ 0x2b, 0x03ff, "Locate (sequential-access)" },
	{ 0x2c, 0x03ff, "Erase (optical-memory)" },
	{ 0x2d, 0x03ff, "Read Updated Block (optical-memory)" },
	{ 0x2e, 0x03ff, "Write with Verify" },
	{ 0x2f, 0x03ff, "Verify" },
	{ 0x30, 0x03ff, "Search Data High" },
	{ 0x31, 0x03ff, "Search Data Equal" },
	{ 0x32, 0x03ff, "Search Data Low" },
	{ 0x33, 0x03ff, "Set Limits" },
	{ 0x34, 0x03ff, "Pre-fetch (pre-read into cache)" },
	{ 0x34, 0x03ff, "Read position (sequential-access device)" },
	{ 0x35, 0x03ff, "Synchronize Cache" },
	{ 0x36, 0x03ff, "Loca/Unlock Cache" },
	{ 0x37, 0x03ff, "Read Defect Data" },
	{ 0x38, 0x03ff, "Medium Scan (optical-memory)" },
	{ 0x39, 0x03ff, "Compare" },
	{ 0x3a, 0x03ff, "Copy and Verify" },
	{ 0x3b, 0x03ff, "Write Buffer" },
	{ 0x3c, 0x03ff, "Read Buffer" },
	{ 0x3d, 0x03ff, "Update Block (optical-memory)" },
	{ 0x3e, 0x03ff, "Read long" },
	{ 0x3f, 0x03ff, "Write long" },
	{ 0x40, 0x03ff, "Change definition (SCSI-1, SCSI-2, CCS)" },
	{ 0x41, 0x03ff, "Write same" },
	{ 0x42, 0x03ff, "Read Sub-Channel (cd-rom)" },
	{ 0x43, 0x03ff, "Read TOC (cd-rom)" },
	{ 0x44, 0x03ff, "Read Header (cd-rom)" },
	{ 0x45, 0x03ff, "Play Audio(10) (cd-rom)" },
	{ 0x46, 0x03ff, "(reserved)" },
	{ 0x47, 0x03ff, "Play Audio MSF (cd-rom)" },
	{ 0x48, 0x03ff, "Play Audio Track Index (cd-rom)" },
	{ 0x49, 0x03ff, "Play Track Relative(10) (cd-rom)" },
	{ 0x4a, 0x03ff, "(reserved)" },
	{ 0x4b, 0x03ff, "Pause Resume (cd-rom)" },
	{ 0x4c, 0x03ff, "Log Select" },
	{ 0x4d, 0x03ff, "Log Sense" },
	{ 0x4e, 0x03ff, "(reserved)" },
	{ 0x4f, 0x03ff, "(reserved)" },
	{ 0x55, 0x03ff, "Mode Select" },
	{ 0x5a, 0x03ff, "Mode Sense" },
	{ 0xa0, 0x03ff, "(reserved)" },
	{ 0xa1, 0x03ff, "(reserved)" },
	{ 0xa2, 0x03ff, "(reserved)" },
	{ 0xa3, 0x03ff, "(reserved)" },
	{ 0xa4, 0x03ff, "(reserved)" },
	{ 0xa5, 0x03ff, "Move Medium (media-changer)" },
	{ 0xa5, 0x03ff, "Play Audio (cd-rom)" },
	{ 0xa6, 0x03ff, "Exchange Medium (media-changer)" },
	{ 0xa7, 0x03ff, "(reserved)" },
	{ 0xa8, 0x03ff, "Get Message (communications device)" },
	{ 0xa8, 0x03ff, "Read (cd-rom)" },
	{ 0xa9, 0x03ff, "Play Track Relative (cd-rom)" },
	{ 0xaa, 0x03ff, "Send Message (communications device)" },
	{ 0xaa, 0x03ff, "Write (optical-memory)" },
	{ 0xab, 0x03ff, "(reserved)" },
	{ 0xac, 0x03ff, "Erase (optical-memory)" },
	{ 0xad, 0x03ff, "(reserved)" },
	{ 0xae, 0x03ff, "Write and Verify (optical-memory)" },
	{ 0xaf, 0x03ff, "Verify (optical-memory)" },
	{ 0xb0, 0x03ff, "Search Data High (cd-rom)" },
	{ 0xb1, 0x03ff, "Search Data Equal (cd-rom)" },
	{ 0xb2, 0x03ff, "Search Data Low (cd-rom)" },
	{ 0xb3, 0x03ff, "Set Limits (cd-rom)" },
};

const struct scsi_types {
	int				peripheral;	/* Peripheral type */
	unsigned		type;		/* Device types */
	const char		*desc;		/* Command description */
} scsi_types[] = {
	{ 0x00, 0x0001, "Direct-Access" },
	{ 0x01, 0x0002, "Sequential-Access" },
	{ 0x02, 0x0004, "Printer" },
	{ 0x03, 0x0008, "Processor" },
	{ 0x04, 0x0010, "WORM" },
	{ 0x05, 0x0020, "CD-ROM" },
	{ 0x06, 0x0040, "Scanner" },
	{ 0x07, 0x0080, "Optical" },
	{ 0x08, 0x0100, "Medium_Changer" },
	{ 0x09, 0x0200, "Communications" },
	{ 0x1F, 0x0000, "Unknown" },
};
